package exnihilocreatio.tiles;

import exnihilocreatio.config.ModConfig;
import exnihilocreatio.rotationalPower.IRotationalPowerConsumer;
import exnihilocreatio.rotationalPower.IRotationalPowerMember;
import exnihilocreatio.util.BlockInfo;
import exnihilocreatio.util.Util;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ITickable;
import net.minecraft.util.Tuple;
import net.minecraft.util.math.BlockPos;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import net.minecraftforge.items.CapabilityItemHandler;

import javax.annotation.Nonnull;
import javax.vecmath.Point3f;
import java.util.ArrayList;
import java.util.List;

public class TileAutoSifter extends BaseTileEntity implements ITickable, IRotationalPowerConsumer {
    public TileSieve[][] toSift = null;
    public List<Tuple<Point3f, EnumFacing.Axis>> connectionPieces = new ArrayList<>();

    public EnumFacing facing = EnumFacing.NORTH;
    public int tickCounter = 0;
    public float rotationValue = 0;
    public float perTickRotation = 0;

    public float storedRotationalPower = 0;

    public float offsetX = 0;
    public float offsetY = 0;
    public float offsetZ = 0;

    public ItemHandlerAutoSifter itemHandlerAutoSifter;

    public TileAutoSifter() {
        itemHandlerAutoSifter = new ItemHandlerAutoSifter();
        itemHandlerAutoSifter.setTe(this);
    }

    @Override
    public void func_73660_a() {
        tickCounter++;

        if (field_145850_b.field_72995_K && perTickRotation != 0) {
            float r = 0.1F;
            float cx = 0F;

            offsetX = cx + r * (float) Math.cos(tickCounter);
        }

        if (tickCounter % 10 == 0) {
            perTickRotation = calcEffectivePerTickRotation(facing);

            BlockPos posOther = field_174879_c.func_177984_a();
            TileEntity te = field_145850_b.func_175625_s(posOther);

            if (te != null && te instanceof TileSieve) {
                toSift = collectPossibleSieves((TileSieve) te);
            } else {
                toSift = null;
            }
        }

        storedRotationalPower += perTickRotation;

        if (Math.abs(storedRotationalPower) > 100 && toSift != null && !field_145850_b.field_72995_K) {
            storedRotationalPower += storedRotationalPower > 0 ? -100 : 100;
            doAutoSieving(toSift);
        }


        if (field_145850_b.field_72995_K) {
            rotationValue += perTickRotation;

            if (tickCounter % 10 == 0) {
                calculateConnectionPieces();
            }
        }
    }

    private void calculateConnectionPieces() {
        connectionPieces.clear();

        if (toSift != null) {
            for (int x = 0; x < toSift.length; x++) {
                for (int z = 0; z < toSift.length; z++) {
                    if (toSift[x][z] != null) {
                        if (x + 1 < toSift.length) {
                            if (toSift[x + 1][z] != null) {
                                connectionPieces.add(new Tuple<>(new Point3f(x + 0.5F - ModConfig.sieve.autoSieveRadius, 0.3F, z - ModConfig.sieve.autoSieveRadius), EnumFacing.Axis.X));
                            }
                        }
                        if (z + 1 < toSift.length) {
                            if (toSift[x][z + 1] != null) {
                                connectionPieces.add(new Tuple<>(new Point3f(x - ModConfig.sieve.autoSieveRadius, 0.3F, z + 0.5F - ModConfig.sieve.autoSieveRadius), EnumFacing.Axis.Z));
                            }
                        }

                    }
                }
            }
        }
    }


    private TileSieve[][] collectPossibleSieves(TileSieve thisSieve) {
        BlockPos sievePos = thisSieve.func_174877_v();

        TileSieve[][] sieveMap = new TileSieve[ModConfig.sieve.autoSieveRadius * 2 + 1][ModConfig.sieve.autoSieveRadius * 2 + 1];

        for (int xOffset = -1 * ModConfig.sieve.autoSieveRadius; xOffset <= ModConfig.sieve.autoSieveRadius; xOffset++) {
            for (int zOffset = -1 * ModConfig.sieve.autoSieveRadius; zOffset <= ModConfig.sieve.autoSieveRadius; zOffset++) {
                sieveMap[xOffset + ModConfig.sieve.autoSieveRadius][zOffset + ModConfig.sieve.autoSieveRadius] = null;

                TileEntity entity = field_145850_b.func_175625_s(sievePos.func_177982_a(xOffset, 0, zOffset));
                if (isValidPartnerSieve(thisSieve, entity)) {
                    sieveMap[xOffset + ModConfig.sieve.autoSieveRadius][zOffset + ModConfig.sieve.autoSieveRadius] = (TileSieve) entity;
                }

            }
        }

        return sieveMap;
    }

    private boolean isValidPartnerSieve(TileSieve thisSieve, TileEntity tileOther) {
        if (tileOther != null && tileOther instanceof TileSieve) {

            TileSieve sieve = (TileSieve) tileOther;
            sieve.validateAutoSieve();

            if (sieve.autoSifter == this) {
                return true;
            } else if (sieve.autoSifter == null) {
                sieve.autoSifter = this;
                return true;
            }
        }
        return false;
    }

    private void doAutoSieving(TileSieve[][] sieveMap) {
        for (TileSieve[] tileSieves : sieveMap) {
            for (TileSieve tileSieve : tileSieves) {
                if (tileSieve != null) {
                    if (!itemHandlerAutoSifter.getStackInSlot(0).func_190926_b() && tileSieve.addBlock(itemHandlerAutoSifter.getStackInSlot(0))) {
                        itemHandlerAutoSifter.getStackInSlot(0).func_190918_g(1);
                    }
                    tileSieve.doSieving(null, true);
                }
            }
        }
    }

    @Override
    public float getMachineRotationPerTick() {
        return perTickRotation;
    }

    @Override
    public void setEffectivePerTickRotation(float rotation) {
        perTickRotation = rotation;
    }

    private float calcEffectivePerTickRotation(EnumFacing direction) {
        if (facing == direction) {
            BlockPos posProvider = field_174879_c.func_177972_a(facing.func_176734_d());
            TileEntity te = field_145850_b.func_175625_s(posProvider);
            if (te != null && te instanceof IRotationalPowerMember) {
                return ((IRotationalPowerMember) te).getEffectivePerTickRotation(facing) + getOwnRotation();
            } else return getOwnRotation();
        } else return 0;
    }

    @SuppressWarnings("unchecked")
    @Override
    public <T> T getCapability(@Nonnull Capability<T> capability, EnumFacing facing) {
        if (capability == CapabilityItemHandler.ITEM_HANDLER_CAPABILITY) {
            return (T) itemHandlerAutoSifter;
        }

        return super.getCapability(capability, facing);
    }

    @Override
    public boolean hasCapability(@Nonnull Capability<?> capability, EnumFacing facing) {
        return capability == CapabilityItemHandler.ITEM_HANDLER_CAPABILITY ||
                super.hasCapability(capability, facing);
    }

    @Override
    @Nonnull
    public NBTTagCompound func_189515_b(NBTTagCompound tag) {
        // if (currentItem != null) {
        //     NBTTagCompound currentItemTag = currentItem.writeToNBT(new NBTTagCompound());
        //     tag.setTag("currentItem", currentItemTag);
        // } TODO: implement


        NBTTagCompound itemHandlerTag = itemHandlerAutoSifter.serializeNBT();
        tag.func_74782_a("itemHandler", itemHandlerTag);

        if (facing != null)
            tag.func_74778_a("facing", facing.func_176610_l());

        tag.func_74776_a("rot", rotationValue);
        tag.func_74776_a("sRot", storedRotationalPower);

        return super.func_189515_b(tag);
    }

    @Override
    public void func_145839_a(NBTTagCompound tag) {
        // if (tag.hasKey("currentItem")) {
        //     currentItem = ItemInfo.readFromNBT(tag.getCompoundTag("currentItem"));
        // } else {
        //     currentItem = null;
        // } TODO: see above

        if (tag.func_74764_b("itemHandler")) {
            itemHandlerAutoSifter.deserializeNBT((NBTTagCompound) tag.func_74781_a("itemHandler"));
        }

        if (tag.func_74764_b("facing"))
            facing = EnumFacing.func_176739_a(tag.func_74779_i("facing"));

        if (tag.func_74764_b("rot"))
            rotationValue = tag.func_74760_g("rot");

        if (tag.func_74764_b("sRot"))
            storedRotationalPower = tag.func_74760_g("sRot");


        super.func_145839_a(tag);
    }

    @SideOnly(Side.CLIENT)
    public TextureAtlasSprite getTexture() {
        if (!itemHandlerAutoSifter.getStackInSlot(0).func_190926_b()) {
            return Util.getTextureFromBlockState(new BlockInfo(itemHandlerAutoSifter.getStackInSlot(0)).getBlockState());
            //TODO: MOVE BLOCKINFO OUT -> should not get recreated each frame
        }

        return null;
    }
}
