package exnihilocreatio.tiles;

import exnihilocreatio.blocks.BlockInfestedLeaves;
import exnihilocreatio.config.ModConfig;
import exnihilocreatio.texturing.Color;
import exnihilocreatio.util.Util;
import lombok.Getter;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.ITickable;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.biome.BiomeColorHelper;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import javax.annotation.Nonnull;

public class TileInfestedLeaves extends BaseTileEntity implements ITickable {
    private static int tileId = 0;

    @Getter
    private float progress = 0;
    @Getter
    private boolean hasNearbyLeaves = true;
    @Getter
    private IBlockState leafBlock = Blocks.field_150362_t.func_176223_P();

    // Stop ALL infested leaves from updating on the same tick always - this way they're evenly spread out and not causing a spike in tick time every time they update
    // Let's hope no one gets 2 billion in their server
    private int updateIndex = tileId++ % ModConfig.infested_leaves.leavesUpdateFrequency;

    @Override
    public void func_73660_a() {
        if (progress < 1.0F) {
            progress += 1.0 / ModConfig.infested_leaves.ticksToTransform;

            markDirtyClient();

            if (progress > 1.0F) {
                progress = 1.0F;

                func_145831_w().func_184138_a(field_174879_c, func_145831_w().func_180495_p(field_174879_c), func_145831_w().func_180495_p(field_174879_c), 3);
            }
        }

        // Don't update unless there's leaves nearby, or we haven't checked for leavesUpdateFrequency ticks. And only update on the server
        if (!func_145831_w().field_72995_K && hasNearbyLeaves || func_145831_w().func_82737_E() % ModConfig.infested_leaves.leavesUpdateFrequency == updateIndex) {
            hasNearbyLeaves = false;

            for (int x = -1; x <= 1; x++) {
                for (int y = -1; y <= 1; y++) {
                    for (int z = -1; z <= 1; z++) {
                        BlockPos newPos = new BlockPos(field_174879_c.func_177982_a(x, y, z));
                        IBlockState state = func_145831_w().func_180495_p(newPos);

                        if (state != Blocks.field_150350_a.func_176223_P() && state.func_177230_c() != Blocks.field_150350_a && (state.func_177230_c() == Blocks.field_150362_t || state.func_177230_c() == Blocks.field_150361_u)) {
                            hasNearbyLeaves = true;

                            if (func_145831_w().field_73012_v.nextFloat() < ModConfig.infested_leaves.leavesSpreadChance) {
                                BlockInfestedLeaves.infestLeafBlock(func_145831_w(), newPos);
                            }
                        }
                    }
                }
            }
        }
    }

    @Override
    @SideOnly(Side.CLIENT)
    public double func_145833_n() {
        return 128 * 128;
    }

    @SideOnly(Side.CLIENT)
    public int getColor() {
        if (field_174879_c == null) {
            return Util.whiteColor.toInt();
        } else {
            Color green = new Color(BiomeColorHelper.func_180287_b(func_145831_w(), field_174879_c));
            return Color.average(green, Util.whiteColor, (float) Math.pow(progress, 2)).toInt();
        }
    }

    public void setProgress(float newProgress) {
        progress = newProgress;
        func_70296_d();
    }

    public void setLeafBlock(IBlockState block) {
        leafBlock = block;
        func_70296_d();
    }

    @Override
    @Nonnull
    public NBTTagCompound func_189515_b(NBTTagCompound tag) {
        tag.func_74776_a("progress", progress);
        tag.func_74778_a("leafBlock", leafBlock.func_177230_c().getRegistryName() == null ? "" : leafBlock.func_177230_c().getRegistryName().toString());
        tag.func_74768_a("leafBlockMeta", leafBlock.func_177230_c().func_176201_c(leafBlock));
        return super.func_189515_b(tag);
    }

    @SuppressWarnings("deprecation")
    @Override
    public void func_145839_a(NBTTagCompound tag) {
        progress = tag.func_74760_g("progress");

        if (tag.func_74764_b("leafBlock") && tag.func_74764_b("leafBlockMeta")) {
            try {
                leafBlock = Block.func_149684_b(tag.func_74779_i("leafBlock")).func_176203_a(tag.func_74762_e("leafBlockMeta"));
            } catch (Exception e) {
                leafBlock = Blocks.field_150362_t.func_176223_P();
            }
        } else {
            leafBlock = Blocks.field_150362_t.func_176223_P();
        }

        super.func_145839_a(tag);
    }
}
